using LiteDB;
using skydiveLogs_api.Domain;
using skydiveLogs_api.DomainService.Repositories;
using skydiveLogs_api.Infrastructure.Interfaces;
using System;
using System.Collections.Generic;

namespace skydiveLogs_api.Infrastructure
{
    public class TunnelFlightRepository : ITunnelFlightRepository
    {
        #region Public Constructors

        public TunnelFlightRepository(IDataProvider dataProvider)
        {
            _dataProvider = dataProvider;
            _col = _dataProvider.CollOfTunnelFlight;
        }

        #endregion Public Constructors

        #region Public Methods

        public int Add(TunnelFlight newTunnelFlight)
        {
            int result;

            try
            {
                var tmp = _col.Insert(newTunnelFlight);
                result = tmp.AsInt32;
            }
            catch
            {
                result = 0;
            }

            return result;
        }

        public bool DeleteById(int id)
        {
            return _col.Delete(new BsonValue(id));
        }

        public IEnumerable<TunnelFlight> GetAll(User user)
        {
            return _col.Include(x => x.Tunnel)
                       .Include(x => x.JumpType)
                       .Find(j => j.User.Id == user.Id);
        }

        public IEnumerable<TunnelFlight> GetAll()
        {
            throw new System.NotImplementedException();
        }

        public IEnumerable<TunnelFlight> GetBetweenIndex(User user, int beginIndex, int endIndex)
        {
            return _col.Include(x => x.Tunnel)
                       .Include(x => x.JumpType)
                       .Query()
                       .OrderByDescending(j => j.FlightDate)
                       .Where(j => j.User.Id == user.Id)
                       .Limit(endIndex - beginIndex)
                       .Offset(beginIndex)
                       .ToList();
        }

        public IEnumerable<TunnelFlight> GetBetweenDate(User user, DateTime beginDate, DateTime endDate)
        {
            return _col.Include(x => x.Tunnel)
                       .Include(x => x.JumpType)
                       .Query()
                       .OrderByDescending(j => j.FlightDate)
                       .Where(j => j.FlightDate >= beginDate)
                       .Where(j => j.FlightDate <= endDate)
                       .ToList();
        }

        public TunnelFlight GetById(int id)
        {
            return _col.FindById(new BsonValue(id));
        }

        public int GetCount(User user)
        {
            return _col.Count(j => j.User.Id == user.Id);
        }

        public int GetCount()
        {
            throw new System.NotImplementedException();
        }

        public bool Update(TunnelFlight updatedTunnelFlight)
        {
            return _col.Update(updatedTunnelFlight);
        }

        #endregion Public Methods

        #region Private Fields

        private readonly ILiteCollection<TunnelFlight> _col;
        private readonly IDataProvider _dataProvider;

        #endregion Private Fields
    }
}